VERSION 5.00
Begin VB.UserControl Dots 
   BackColor       =   &H00FFFFFF&
   BorderStyle     =   1  'Fixed Single
   ClientHeight    =   3795
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   4800
   DrawMode        =   7  'Invert
   ScaleHeight     =   3795
   ScaleWidth      =   4800
End
Attribute VB_Name = "Dots"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'------------------------------------------------------------------------
'   ActiveX Control - Dots.ocx
'It's provides ability to open, view, edit and save
'graphic presentation of data.
'Data is kept in DB layout and presented the array of
'pair coordinates (X, Y) of points.

'   Public Properties:
'DotColor   as OLE_COLOR
'BackColor  as OLE_COLOR
'DotSize    as Integer

'   Public Methods:
'Load (sConnectionString As String) - creates the connection, opens transaction
'                                     and fills the array with coordinates
'Save - commits transaction
'Cancel - rollback transaction
'Refresh - refreshes the control and synchronizes with DB
'
'   Public Event:
'Changed (bDirty) - raises event when changes happened
'--------------------------------------------------------------------------

Option Explicit

'Public Events
Public Event Changed(ByVal bDirty As Boolean)

'Private properties
Private m_connDots      As ADODB.Connection
Private m_arrDots()     As Single   'Array of dots
Private m_nDots         As Integer  'Number of dots
Private m_nCurDot       As Integer  'Number of the moving dot
Private m_nStep         As Integer  '1/2 Size of the dot
Private m_cBackColor    As OLE_COLOR      'Background color
Private m_cDotColor     As OLE_COLOR      'Dots color
Private m_bDirty        As Boolean

Private Sub UserControl_Terminate()
    If Not m_connDots Is Nothing Then Set m_connDots = Nothing
End Sub

Private Sub UserControl_InitProperties()
    DotSize = 60
    m_cBackColor = UserControl.BackColor
    DotColor = vbRed
End Sub

Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
    With PropBag
        DotSize = .ReadProperty("DotSize", 60)
        m_cBackColor = .ReadProperty("BackColor", UserControl.BackColor)
        DotColor = .ReadProperty("DotColor", vbRed)
    End With
End Sub

Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
    With PropBag
        .WriteProperty "DotSize", m_nStep * 2, 60
        .WriteProperty "BackColor", m_cBackColor, UserControl.BackColor
        .WriteProperty "DotColor", m_cDotColor, vbRed
    End With
End Sub

' Custom properties ------------------------------------------
Public Property Get DotSize() As Integer
    DotSize = m_nStep * 2
End Property

Public Property Let DotSize(ByVal vNewValue As Integer)
    If vNewValue <= 0 Then Exit Property
    m_nStep = CInt(vNewValue / 2)
    If UserControl.Ambient.UserMode Then
        UserControl.Refresh
    End If
End Property

Public Property Get BackColor() As OLE_COLOR
    BackColor = m_cBackColor
End Property

Public Property Let BackColor(ByVal vNewValue As OLE_COLOR)
    m_cBackColor = vNewValue
    If UserControl.Ambient.UserMode Then
        UserControl.Refresh
    End If
End Property

Public Property Get DotColor() As OLE_COLOR
    DotColor = m_cDotColor
End Property

Public Property Let DotColor(ByVal vNewValue As OLE_COLOR)
    m_cDotColor = vNewValue
    If UserControl.Ambient.UserMode Then
        UserControl.Refresh
    End If
End Property

'Custom methods ---------------------------------------------
'Method Load
Public Function Load(sConnectionString As String) As Boolean
    Load = False
    
    'Close connection - if exists
    If Not m_connDots Is Nothing Then Set m_connDots = Nothing
    
    'Open Connection
    On Error GoTo err_SQL
    Set m_connDots = CreateObject("ADODB.Connection")
    m_connDots.ConnectionString = sConnectionString
    m_connDots.IsolationLevel = adXactBrowse
    m_connDots.CommandTimeout = 15
    m_connDots.Open
    On Error GoTo 0
    
    'Begin Transaction
    m_connDots.BeginTrans
    
    'Insert coordinates into the m_arrDots
    FillArray
    
    'Number of the moving dot
    m_nCurDot = -1
    
    Load = True
    Exit Function
    
err_SQL:
    MsgBox Err.Description
    Set m_connDots = Nothing
End Function

'Method - Commit Transaction
Public Sub Save()
    If Not m_connDots Is Nothing Then
        m_connDots.CommitTrans
        SetDirty False
        m_connDots.BeginTrans
    End If
End Sub

'Method - Rollback Transaction
Public Sub Cancel()
    If Not m_connDots Is Nothing Then
        m_connDots.RollbackTrans
        m_connDots.BeginTrans
        'repaint control
        Refresh
    End If
End Sub

'Method - Refresh the control
Public Sub Refresh()
    FillArray
    UserControl.Refresh
End Sub

' Interface -------------------------------------------------
Private Sub UserControl_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    'Find the dot in the array
    Dim i As Integer
    i = 0
    While m_nCurDot = -1 And i < m_nDots
        If X >= (m_arrDots(0, i) - m_nStep) And _
        X <= (m_arrDots(0, i) + m_nStep) And _
        Y >= (m_arrDots(1, i) - m_nStep) And _
        Y <= (m_arrDots(1, i) + m_nStep) Then m_nCurDot = i
        i = i + 1
    Wend
End Sub

Private Sub UserControl_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
    If Not m_connDots Is Nothing Then
        If m_nCurDot <> -1 And DotInField(X, Y) Then    'dot is moving in the control
            'Erase the dot
            UserControl.DrawMode = vbXorPen
            DrawDot m_arrDots(0, m_nCurDot), m_arrDots(1, m_nCurDot)
            'Save new coordinates
            m_arrDots(0, m_nCurDot) = X
            m_arrDots(1, m_nCurDot) = Y
            'Draw the dot on the new place
            DrawDot m_arrDots(0, m_nCurDot), m_arrDots(1, m_nCurDot)
        End If
    End If
End Sub

Private Sub UserControl_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
    If Not m_connDots Is Nothing Then
        'Update DB
        If m_nCurDot <> -1 Then
            Dim rst As ADODB.Recordset
            Set rst = CreateObject("ADODB.Recordset")
            rst.Open "SELECT * FROM Dots WHERE DotID=" + CStr(m_arrDots(2, m_nCurDot)), m_connDots, adOpenStatic, adLockPessimistic
            If Not rst.EOF Then
                rst.Fields("X").Value = m_arrDots(0, m_nCurDot)
                rst.Fields("Y").Value = m_arrDots(1, m_nCurDot)
                rst.Update
            End If
            rst.Close
            Set rst = Nothing
            
            SetDirty True
            m_nCurDot = -1
        End If
    End If
End Sub

Private Sub UserControl_Paint()
    'Draw dots
    Dim i As Integer
    UserControl.DrawMode = vbCopyPen
    UserControl.BackColor = m_cBackColor
    For i = 0 To m_nDots - 1
        If DotInField(m_arrDots(0, i), m_arrDots(1, i)) Then _
            DrawDot m_arrDots(0, i), m_arrDots(1, i)
    Next i
End Sub

' Private methods -------------------------------------------
'Check if the dot is on the control
Private Function DotInField(X As Single, Y As Single) As Boolean
    DotInField = False
    If (X - m_nStep) > ScaleLeft And _
        (X + m_nStep) < ScaleWidth And _
        (Y - m_nStep) > ScaleTop And _
        (Y + m_nStep) < ScaleHeight Then DotInField = True
End Function

'Draw dot
Private Sub DrawDot(X As Single, Y As Single)
    Dim clr As OLE_COLOR
    clr = m_cDotColor
    If UserControl.DrawMode = vbXorPen Then _
        clr = m_cDotColor Xor m_cBackColor
    UserControl.Line (X - m_nStep, Y - m_nStep)-(X + m_nStep, Y + m_nStep), _
        clr, BF
End Sub

'Fill array of dots
Private Sub FillArray()
    'Open recordset
    On Error GoTo err_Rst
    Dim rstDots As ADODB.Recordset
    Set rstDots = CreateObject("ADODB.Recordset")
    rstDots.Open "SELECT * FROM Dots", m_connDots, adOpenStatic
    If rstDots.EOF Then
        MsgBox "There is no dots in the database", vbInformation, App.Title
        Set rstDots = Nothing
        Exit Sub
    End If
    On Error GoTo 0
    
    'Insert coordinates into the m_arrDots
    Dim i As Integer
    rstDots.MoveFirst
    ReDim m_arrDots(3, 0)
    m_nDots = 0
    While Not rstDots.EOF
        m_nDots = m_nDots + 1
        ReDim Preserve m_arrDots(3, m_nDots)
        m_arrDots(0, m_nDots - 1) = rstDots.Fields("X").Value
        m_arrDots(1, m_nDots - 1) = rstDots.Fields("Y").Value
        m_arrDots(2, m_nDots - 1) = rstDots.Fields("DotID").Value
        rstDots.MoveNext
    Wend
    SetDirty False
    Set rstDots = Nothing
    Exit Sub
    
err_Rst:
    MsgBox Err.Description
End Sub

'Raise Changed event
Private Sub SetDirty(ByVal bDirty As Boolean)
    If m_bDirty <> bDirty Then
        m_bDirty = bDirty
        RaiseEvent Changed(bDirty)
    End If
End Sub
